@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 1129 2021-07-09 07:36:53Z Casey $
+# $Id: __init__.py 1170 2021-07-24 05:36:21Z Casey $
 #
 # Copyright (c) 2019 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Casey $ (last)
-# $Date: 2021-07-09 15:36:53 +0800 (Fri, 09 Jul 2021) $
-# $Revision: 1129 $
+# $Date: 2021-07-24 13:36:21 +0800 (週六, 24 七月 2021) $
+# $Revision: 1170 $
 
 import os
 import sys
@@ -33,6 +33,9 @@ from fabric import Connection as _Connection
 from fabric import connection
 from fabric.connection import opens
 
+from contextlib import contextmanager
+from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeRemainingColumn, TimeElapsedColumn
+
 # patch
 from invoke.runners import Runner
 
@@ -60,17 +63,60 @@ def read_our_stdin(self, input_):
 
 Runner.read_our_stdin = read_our_stdin
 
-def getBuildData(shPath, dataName):
+def getBuildData(shPath, dataName, alternatives=[]):
     with open(shPath, 'r') as f:
         for line in f.readlines():
             lineWithoutSpace = line.lstrip()
             if lineWithoutSpace:
                 firstInLine = lineWithoutSpace[0]
-            if ("%s=" % dataName in line) and (firstInLine != "#" ):
-                index = line.index("=") + 1
-                return line[index:].replace('\r\n', '').strip()
+
+            if firstInLine != "#": # 沒有註解的才會進來
+                if "%s=" % dataName in line:
+                    index = line.index("=") + 1
+                    return line[index:].replace('\r\n', '').strip()
+                elif alternatives: 
+                    for alternative in alternatives:
+                        if "%s=" % alternative in line:
+                            index = line.index("=") + 1
+                            return line[index:].replace('\r\n', '').strip()
+
     return None
 
+# https://zh.wikipedia.org/wiki/%E4%B8%93%E7%94%A8%E7%BD%91%E7%BB%9C   
+def isVirtualMachine(ip):
+    if ip.startswith("192.168.") or ip.startswith("10."):
+        return True
+    for i in range(16, 32):
+        if ip.startswith("172.%s." % i):
+            return True
+    return False
+    
+@contextmanager
+def createProgress(name="[red]Processing"):
+    class Task:
+        def __init__(self, progress, task):
+            self.progress = progress
+            self.task = task
+        def update(self, advance):
+            self.progress.update(self.task, advance=advance)
+            
+    if hasattr(createProgress, 'live'):
+        progress = createProgress.live
+        task = progress.add_task(name, total=100)
+        yield Task(progress, task)     
+    else:
+        with Progress(
+            SpinnerColumn(),
+            TextColumn("[progress.description]{task.description}"),
+            BarColumn(),
+            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
+            TimeRemainingColumn(),
+            TimeElapsedColumn(),
+        ) as progress:
+            createProgress.live = progress
+            task = progress.add_task(name, total=100)
+            yield Task(progress, task)  
+
 def showSplash():
     import pyfiglet
     cprint("-" * 79, "white", attrs=["bold"],)
@@ -167,7 +213,7 @@ def record(func, c, *args, **kwargs):
 
 # patch
 @opens
-def run(self, command, notRecord=False, **kwargs):
+def run(self, command, notRecord=False, hideStdout=False, **kwargs):
     """
     Execute a shell command on the remote end of this connection.
 
@@ -193,6 +239,9 @@ def run(self, command, notRecord=False, **kwargs):
 
     if six.PY2 and not kwargs.get("pty"):
         kwargs["hide"] = True
+        
+    if hideStdout:
+        kwargs["hide"] = True
 
     if not notRecord:
         c = "%s: %s" % (command, str(datetime.datetime.now().time()))
